@@ -11,7 +11,11 @@ struct fts_xapian_user_module fts_xapian_user_module = MODULE_CONTEXT_INIT(&mail
 
 static void fts_xapian_mail_user_deinit(struct mail_user *user)
 {
+#if ((DOVECOT_VERSION_MINOR > 2) || (DOVECOT_VERSION_MAJOR > 2))
 	struct fts_xapian_user *fuser = FTS_XAPIAN_USER_CONTEXT_REQUIRE(user);
+#else
+	struct fts_xapian_user *fuser = FTS_XAPIAN_USER_CONTEXT(user);
+#endif
 
         fts_mail_user_deinit(user);
 	fuser->module_ctx.super.deinit(user);
@@ -31,7 +35,7 @@ static void fts_xapian_mail_user_created(struct mail_user *user)
         fuser->set.partial 	= XAPIAN_DEFAULT_PARTIAL;
         fuser->set.full 	= XAPIAN_DEFAULT_FULL;
 
-#ifdef __FreeBSD__
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
 	size_t len = sizeof(fuser->set.pagesize);
 	sysctlbyname("hw.pagesize", &(fuser->set.pagesize), &len, NULL, 0);
 #else
@@ -80,7 +84,7 @@ static void fts_xapian_mail_user_created(struct mail_user *user)
                 	}
                 	else if (strncmp(*tmp,"lowmemory=",10)==0)
                 	{
-                	        len=atol(*tmp + 9);
+                	        len=atol(*tmp + 10);
                 	        if(len>0) { fuser->set.lowmemory = len; }
                 	}
                 	else if (strncmp(*tmp,"attachments=",12)==0)
